import jade.content.lang.*;
import jade.content.lang.sl.*;
import jade.content.onto.*;
import jade.core.*;
import jade.core.behaviours.*;
import jade.lang.acl.*;

public class BuyerAgent extends Agent {
    protected String agentName = "Buyer";
    protected Codec codec = new SLCodec();
    protected Ontology ontology = MusicShopOntology.getInstance();

    protected void setup() {
        super.setup();
        System.out.println(agentName+": Started...");

        getContentManager().registerLanguage(codec);
        getContentManager().registerOntology(ontology);
        System.out.println(agentName+": Ontology registered...");

        addBehaviour(new BuyerBehaviour(this));
        System.out.println(agentName+": Behaviour ran...");
    }

    class BuyerBehaviour extends SimpleBehaviour {
        private static final String SELLER_NAME = "sa@Orange:1099/JADE";

        private boolean finished = false;

        public BuyerBehaviour(Agent agent) {
            super(agent);
        }

        public void action() {
            try {
                AID sellerAID = new AID(SELLER_NAME, true);

                ACLMessage msg = new ACLMessage(ACLMessage.QUERY_IF);
                msg.setSender(getAID());
                msg.addReceiver(sellerAID);
                msg.setLanguage(codec.getName());
                msg.setOntology(ontology.getName());
                System.out.println(agentName+": Message initialized...");

                Book book = new Book();
                book.setAuthor("Martin Fowler");
                book.setName("Extreme Programming");
                System.out.println(agentName+": Object built...");

                Owns owns = new Owns();
                owns.setOwner(sellerAID);
                owns.setItem(book);
                getContentManager().fillContent(msg, owns);
                System.out.println(agentName+": Predicate built...");

                System.out.println(agentName+": Sending request...");
                send(msg);

                System.out.println(agentName+": Waiting for reply...");
                ACLMessage reply = blockingReceive();
                if (reply.getPerformative() == ACLMessage.CONFIRM)
                    System.out.println(agentName+": Received CONFIRM");
                else if (reply.getPerformative() == ACLMessage.REFUSE)
                    System.out.println(agentName+": Received REFUSE");
                else
                    System.out.println(agentName+": Received unknown");

                finished = true;
            }
            catch(Exception e) {
                finished = true;
                e.printStackTrace();
            }
        }

        public boolean done() {
            return finished;
        }
    }
}
