import jade.content.onto.*;
import jade.content.schema.*;

public class MusicShopOntology extends Ontology {
    public static final String ONTOLOGY_NAME = "Music-shop-ontologies";

    public static final String ITEM = "Item";
    public static final String ITEM_PRICE = "price";

    public static final String BOOK = "Book";
    public static final String BOOK_AUTHOR = "author";
    public static final String BOOK_NAME = "name";

    public static final String OWNS = "Owns";
    public static final String OWNS_OWNER = "owner";
    public static final String OWNS_ITEM = "item";

    public static final String SELL = "Sell";
    public static final String SELL_BUYER = "buyer";
    public static final String SELL_ITEM = "item";

    private static Ontology theInstance = new MusicShopOntology();

    public static Ontology getInstance() {
        return theInstance;
    }

    public MusicShopOntology() {
        super(ONTOLOGY_NAME, BasicOntology.getInstance());

        try {
            add(new ConceptSchema(ITEM), Item.class);
            add(new ConceptSchema(BOOK), Book.class);
            add(new PredicateSchema(OWNS), Owns.class);
            add(new AgentActionSchema(SELL), Sell.class);

            ConceptSchema cs = (ConceptSchema) getSchema(ITEM);
            cs.add(ITEM_PRICE, (PrimitiveSchema) getSchema(BasicOntology.FLOAT), ObjectSchema.OPTIONAL);

            cs = (ConceptSchema) getSchema(BOOK);
            cs.addSuperSchema((ConceptSchema) getSchema(ITEM));
            cs.add(BOOK_AUTHOR, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            cs.add(BOOK_NAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));

            PredicateSchema ps = (PredicateSchema) getSchema(OWNS);
            ps.add(OWNS_OWNER, getSchema(BasicOntology.AID));
            ps.add(OWNS_ITEM, getSchema(ITEM));

            AgentActionSchema as = (AgentActionSchema) getSchema(SELL);
            as.add(SELL_BUYER, (ConceptSchema) getSchema(BasicOntology.AID));
            as.add(SELL_ITEM, (ConceptSchema) getSchema(ITEM));
        } catch (OntologyException e) {
            e.printStackTrace();
        }
    }
}
