import jade.content.*;
import jade.content.lang.*;
import jade.content.lang.sl.*;
import jade.content.onto.*;
import jade.core.*;
import jade.core.behaviours.*;
import jade.lang.acl.*;

public class SellerAgent extends Agent {
    protected String agentName = "Seller";
    protected Codec codec = new SLCodec();
    protected Ontology ontology = MusicShopOntology.getInstance();

    protected void setup() {
        super.setup();
        System.out.println(agentName+": Started...");

        getContentManager().registerLanguage(codec);
        getContentManager().registerOntology(ontology);
        System.out.println(agentName+": Ontology registered...");

        addBehaviour(new SellerBehaviour(this));
        System.out.println(agentName+": Behaviour ran...");
    }

    class SellerBehaviour extends SimpleBehaviour {
        private static final String BUYER_NAME = "ba@Orange:1099/JADE";

        private boolean finished = false;

        public SellerBehaviour(Agent agent) {
            super(agent);
        }

        public void action() {
            try {
                AID buyerAID = new AID(BUYER_NAME, true);

                System.out.println(agentName+": Waiting for message...");
                ACLMessage msg = blockingReceive();
                System.out.println(agentName+": Received "+msg.getContent());

                if (msg.getPerformative() == ACLMessage.QUERY_IF) {
                    System.out.println(agentName+": Message is of type QUERY_IF...");
                    ContentElement element = getContentManager().extractContent(msg);
                    if (element instanceof Owns) {
                        System.out.println(agentName+": Message contains predicate OWNS...");
                        Owns owns = (Owns)element;
                        Item item = owns.getItem();
                        if (item instanceof Book) {
                            System.out.println(agentName+": Predicate contains item of type BOOK...");
                            Book book = (Book)item;
                            ACLMessage reply;
                            if (book.getAuthor().equals("Martin Fowler"))
                                reply = new ACLMessage(ACLMessage.CONFIRM);
                            else
                                reply = new ACLMessage(ACLMessage.REFUSE);
                            reply.setSender(getAID());
                            reply.addReceiver(buyerAID);
                            System.out.println(agentName+": Reply built...");

                            System.out.println(agentName+": Sending reply...");
                            send(reply);
                        }
                    }
                }
                finished = true;
            } catch(Exception e) {
                finished = true;
                e.printStackTrace();
            }
        }

        public boolean done() {
            return finished;
        }
    }
}
