using System;
using System.Collections.Generic;
using System.Text;

using jade.core;
using jade.core.behaviours;

using log4net;

using Mailabs.Reasoning;
using System.Collections;

namespace Mailabs.Tutorial.MAS
{
    public class CommAgent : Agent
    {
        private static readonly ILog logger = LogManager.GetLogger(typeof(CommAgent));

        internal ReasoningEntity profile;
        internal ArrayList friends = new ArrayList();

        public override void setup()
        {
            logger.Debug("Setting up communication agent " + getLocalName() + "...");

            object[] args = getArguments();

            if (args != null && args.Length > 0)
            {
                logger.Debug("Reading " + args.Length + " received arguments");

                for (int i = 0; i < args.Length; i++)
                {
                    logger.Debug("Passed: " + args[i]);
                    if (args[i] is string)
                    {
                        string argsi = (string)args[i];
                        if (argsi.IndexOf('=') != -1)
                        {
                            string[] parts = argsi.Split(new char[] { '=' });
                            if (parts.Length != 2)
                                continue;

                            if (parts[0].Equals(Consts.PROFILE_URI_PARAM) && profile == null)
                            {
                                logger.Debug("Loading profile from: " + parts[1]);
                                Ontology profileModel = CacheService.CacheFor(parts[1]);
                                logger.Debug("Profile model loaded, trying to fetch root class with URI: " + GetProfileRootUri(profileModel.Uri, getLocalName()));
                                profile = new ReasoningEntity(profileModel.Classes[GetProfileRootUri(profileModel.Uri, getLocalName())]);
                                logger.Debug("Profile loaded successfully, the root is " + profile.Root);
                            }

                            if (parts[0].Equals(Consts.FRIEND_NAME_PARAM))
                            {
                                logger.Debug("Found this agent's (" + getLocalName() + ") friend, the name is " + parts[1] + ", registering");
                                friends.Add(parts[1]);
                            }
                        }
                    }
                }

                string workDir = Consts.AGENT_FOLDERS_PREFIX + getLocalName() + Consts.AGENT_REQUESTS_FOLDER_SUFFIX;
                string processedDir = Consts.AGENT_FOLDERS_PREFIX + getLocalName() + Consts.AGENT_PROCESSED_FOLDER_SUFFIX;
                string failedDir = Consts.AGENT_FOLDERS_PREFIX + getLocalName() + Consts.AGENT_FAILED_FOLDER_SUFFIX;

                try
                {
                    System.IO.Directory.CreateDirectory(workDir);
                    System.IO.Directory.CreateDirectory(processedDir);
                    System.IO.Directory.CreateDirectory(failedDir);
                }
                catch (Exception e)
                {
                    logger.Error("Unable to create work folders for the agent " + getLocalName(), e);
                    throw new AgentInitializationException("Unable to create work folders for the agent " + getLocalName(), e);
                }

                if (profile == null || profile.Root == null)
                {
                    logger.Error("Agent's ontoprofile was not properly configured, please check configuration and system log for details");
                    throw new AgentInitializationException("Agent's ontoprofile was not properly configured, please check configuration and system log for details");
                }

                logger.Debug("Setting default communication agent behaviour");
                addBehaviour(new RequestPollingBehaviour(this, workDir, processedDir, failedDir));
                addBehaviour(new MessagePollingBehaviour(this));

                logger.Debug("Setup of communication agent " + getLocalName() + " successfull");
            }
        }

        public override void takeDown()
        {
            string workDir = Consts.AGENT_FOLDERS_PREFIX + getLocalName() + Consts.AGENT_REQUESTS_FOLDER_SUFFIX;
            string processedDir = Consts.AGENT_FOLDERS_PREFIX + getLocalName() + Consts.AGENT_PROCESSED_FOLDER_SUFFIX;
            string failedDir = Consts.AGENT_FOLDERS_PREFIX + getLocalName() + Consts.AGENT_FAILED_FOLDER_SUFFIX;

            try
            {
                System.IO.Directory.Delete(workDir, true);
                System.IO.Directory.Delete(processedDir, true);
                System.IO.Directory.Delete(failedDir, true);
            }
            catch (Exception e)
            {
                logger.Error("Unable to clean up work folders for the agent " + getLocalName(), e);
                throw new AgentInitializationException("Unable to clean up work folders for the agent " + getLocalName(), e);
            }            
        }

        private string GetProfileRootUri(string ontologyUri, string profileName)
        {
            if (ontologyUri.EndsWith("/"))
                return ontologyUri + profileName;
            else if (ontologyUri.EndsWith("#"))
                return ontologyUri + profileName;
            else
                return ontologyUri + '#' + profileName;
        }
    }
}
