using System;
using System.IO;
using System.Collections;
using System.Runtime.Serialization.Formatters.Binary;

using jade.core;
using jade.lang.acl;
using jade.core.behaviours;

using log4net;

using Mailabs.Reasoning;

namespace Mailabs.Tutorial.MAS
{
    public class RequestPollingBehaviour : TickerBehaviour
    {
        private static readonly ILog logger = LogManager.GetLogger(typeof(RequestPollingBehaviour));

        private string workDir;
        private string processedDir;
        private string failedDir;

        private ReasoningEntity request;

        public RequestPollingBehaviour(Agent agent, string workDir, string processedDir, string failedDir) : base(agent, 2000)
        {
            this.workDir = workDir;
            this.processedDir = processedDir;
            this.failedDir = failedDir;
        }

        public override void onTick()
        {
            string[] requests = Directory.GetFiles(workDir, "*.owl", SearchOption.TopDirectoryOnly);
            if (requests == null || requests.Length == 0)
            {
                return;
            }

            logger.Debug("Found new requests for agent, processing");

            DirectoryInfo fullWorkDir = new DirectoryInfo(workDir);
            for (int i = 0; i < requests.Length; i++)
            {
                string requestFileName = requests[i].Substring(workDir.Length);

                try
                {
                    string requestFileUri = "file://" + (fullWorkDir.FullName + requestFileName).Replace('\\', '/').ToLower();
                    logger.Debug("Loading request from: " + requestFileUri);
                    Ontology requestModel = CacheService.CacheFor(requestFileUri);
                    logger.Debug("Request model loaded, trying to fetch request root class with URI: " + GetRequestRootUri(requestModel.Uri, Consts.REQUEST_ROOT_CLASS));
                    request = new ReasoningEntity(requestModel.Classes[GetRequestRootUri(requestModel.Uri, Consts.REQUEST_ROOT_CLASS)]);
                    logger.Debug("Request loaded successfully, the root is " + request.Root);
                    File.Move(workDir + requestFileName, processedDir + requestFileName);

                    request.Weight = 1;
                    RequestProcessor.ProcessRequest((CommAgent)myAgent, request, true);
                }
                catch (Exception e)
                {
                    logger.Warn("Unable to process request (parse error?), moving it to 'failed' folder", e);
                    File.Move(workDir + requestFileName, failedDir + requestFileName);
                }
            }
        }

        private string GetRequestRootUri(string ontologyUri, string profileName)
        {
            if (ontologyUri.EndsWith("/"))
                return ontologyUri + profileName;
            else if (ontologyUri.EndsWith("#"))
                return ontologyUri + profileName;
            else
                return ontologyUri + '#' + profileName;
        }
    }
}
