/*

  ORe - OWL ontology relevance library for Microsoft .NET
  (http://ore.mailabs.ru)

  Copyright (c) 2005, 2006 Sergey V. Mikhanov (MAILabs)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  Version: $Id: $
  
*/

using System;
using System.Collections;

namespace Mailabs.Reasoning
{
    /// <summary>
    ///   OWL-.    , ,     .
    /// </summary>
    [Serializable]
    public class OwlClassDictionary : DictionaryBase
    {
        public OwlClass this[string key]
        {
            get
            {
                return (OwlClass)Dictionary[key];
            }
            set
            {
                Dictionary[key] = value;
            }
        }

        public ICollection Keys
        {
            get
            {
                return Dictionary.Keys;
            }
        }

        public ICollection Values
        {
            get
            {
                return Dictionary.Values;
            }
        }

        public void Add(string key, OwlClass value)
        {
            Dictionary.Add(key, value);
        }

        public bool Contains(string key)
        {
            return Dictionary.Contains(key);
        }

        public void Remove(string key)
        {
            Dictionary.Remove(key);
        }

        protected override void OnInsert(Object key, Object value)
        {
            if (key.GetType() != Type.GetType("System.String"))
                throw new ArgumentException("Key must be of type String.");

            if (value.GetType() != Type.GetType("Mailabs.Reasoning.OwlClass"))
                throw new ArgumentException("Value must be of type OwlClass.");

            if (((OwlClass)value).ID != (string)key) 
                throw new ArgumentException("The key and the ID of the value doesn't match.");
        }

        protected override void OnRemove(Object key, Object value)
        {
            if (key.GetType() != Type.GetType("System.String"))
                throw new ArgumentException("Key must be of type String.");
        }

        protected override void OnSet(Object key, Object oldValue, Object newValue)
        {
            if (key.GetType() != Type.GetType("System.String"))
                throw new ArgumentException("Key must be of type String.");

            if (newValue.GetType() != Type.GetType("Mailabs.Reasoning.OwlClass"))
                throw new ArgumentException("Value must be of type OwlClass.");

            if (((OwlClass)newValue).ID != (string)key) 
                throw new ArgumentException("The key and the ID of the value doesn't match.");
        }

        protected override void OnValidate(Object key, Object value)
        {
            if (key.GetType() != Type.GetType("System.String"))
                throw new ArgumentException("Key must be of type String.");

            if (value.GetType() != Type.GetType("Mailabs.Reasoning.OwlClass"))
                throw new ArgumentException("Value must be of type OwlClass.");

            if (((OwlClass)value).ID != (string)key) 
                throw new ArgumentException("The key and the ID of the value doesn't match.");
        }
    }

    /// <summary>
    ///   OWL-.    , ,     .
    /// </summary>
    [Serializable]
    public class OwlRoleDictionary : DictionaryBase
    {
        public OwlRole this[string key]
        {
            get
            {
                return (OwlRole)Dictionary[key];
            }
            set
            {
                Dictionary[key] = value;
            }
        }

        public ICollection Keys
        {
            get
            {
                return Dictionary.Keys;
            }
        }

        public ICollection Values
        {
            get
            {
                return Dictionary.Values;
            }
        }

        public void Add(string key, OwlRole value)
        {
            Dictionary.Add(key, value);
        }

        public bool Contains(string key)
        {
            return Dictionary.Contains(key);
        }

        public void Remove(string key)
        {
            Dictionary.Remove(key);
        }

        protected override void OnInsert(Object key, Object value)
        {
            if (key.GetType() != Type.GetType("System.String"))
                throw new ArgumentException("Key must be of type String.");

            if (value.GetType() != Type.GetType("Mailabs.Reasoning.OwlRole"))
                throw new ArgumentException("Value must be of type OwlRole.");

            if (((OwlRole)value).ID != (string)key) 
                throw new ArgumentException("The key and the ID of the value doesn't match.");
        }

        protected override void OnRemove(Object key, Object value)
        {
            if (key.GetType() != Type.GetType("System.String"))
                throw new ArgumentException("Key must be of type String.");
        }

        protected override void OnSet(Object key, Object oldValue, Object newValue)
        {
            if (key.GetType() != Type.GetType("System.String"))
                throw new ArgumentException("Key must be of type String.");

            if (newValue.GetType() != Type.GetType("Mailabs.Reasoning.OwlRole"))
                throw new ArgumentException("Value must be of type OwlRole.");

            if (((OwlRole)newValue).ID != (string)key) 
                throw new ArgumentException("The key and the ID of the value doesn't match.");
        }

        protected override void OnValidate(Object key, Object value)
        {
            if (key.GetType() != Type.GetType("System.String"))
                throw new ArgumentException("Key must be of type String.");

            if (value.GetType() != Type.GetType("Mailabs.Reasoning.OwlRole"))
                throw new ArgumentException("Value must be of type OwlRole.");

            if (((OwlRole)value).ID != (string)key) 
                throw new ArgumentException("The key and the ID of the value doesn't match.");
        }
    }
}
