/*

  ORe - OWL ontology relevance library for Microsoft .NET
  (http://ore.mailabs.ru)

  Copyright (c) 2005, 2006 Sergey V. Mikhanov (MAILabs)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  Version: $Id: $
  
*/

using System;

namespace Mailabs.Reasoning
{
    public abstract class LoadCallback
    {
        public abstract void OnCacheInit();
        public abstract void OnCacheCreate();
        public abstract void OnHelperInit();
        public abstract void OnBeforeParse();
        public abstract void OnAfterParse(int nodesCount);
        public abstract void OnNodeLoad(int nodeNumber);
        public abstract void OnError(Exception e);
        public abstract void OnValidationEvent(string message);

        public static readonly LoadCallback NULL = new NullLoadCallback();
    }

    public class NullLoadCallback : LoadCallback 
    {
        public override void OnCacheInit()
        {

        }

        public override void OnCacheCreate()
        {

        }

        public override void OnHelperInit()
        {

        }

        public override void OnBeforeParse()
        {

        }

        public override void OnAfterParse(int nodesCount)
        {
        
        }

        public override void OnNodeLoad(int nodeNumber)
        {

        }

        public override void OnError(Exception e)
        {
            
        }

        public override void OnValidationEvent(string message)
        {

        }
    }

    public class PrintLoadCallback : LoadCallback 
    {
        public override void OnCacheInit()
        {
            Console.WriteLine("Cache Init done " + DateTime.Now.ToString("HH:mm:ss.fff"));
        }

        public override void OnCacheCreate()
        {
            Console.WriteLine("Cache create started " + DateTime.Now.ToString("HH:mm:ss.fff"));
        }

        public override void OnHelperInit()
        {
            Console.WriteLine("Helper Init done " + DateTime.Now.ToString("HH:mm:ss.fff"));
        }

        public override void OnBeforeParse()
        {
            Console.WriteLine("Parsing started " + DateTime.Now.ToString("HH:mm:ss.fff"));
        }

        public override void OnAfterParse(int nodesCount)
        {
            Console.WriteLine("Parsing done, nodes found: " + nodesCount + " @ " + DateTime.Now.ToString("HH:mm:ss.fff"));
        }

        public override void OnNodeLoad(int nodeNumber)
        {
            Console.WriteLine("Loading node: " + nodeNumber + " @ " + DateTime.Now.ToString("HH:mm:ss.fff"));
        }

        public override void OnError(Exception e)
        {
            Console.WriteLine("Error: " + e.Message);
        }

        public override void OnValidationEvent(string message)
        {
            Console.WriteLine("Validation event: " + message);
        }
    }
}
