/*

  ORe - OWL ontology relevance library for Microsoft .NET
  (http://ore.mailabs.ru)

  Copyright (c) 2005, 2006 Sergey V. Mikhanov (MAILabs)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  Version: $Id: $
  
*/

using System;
using System.Collections;

namespace Mailabs.Reasoning
{
    /// <summary>
    ///  OWL- . , ,  .
    /// </summary>
    /// <seealso cref="OwlDisjunctiveClass"/>
    [Serializable]
    public class OwlClass
    {
        public readonly string ID;

        public readonly OwlClassDictionary Parents;
        public readonly OwlClassDictionary Equivalents;
        public readonly OwlClassDictionary Complements;
        public readonly OwlRestrictionCollection Restrictions;

        public readonly bool IsRemote;

        public OwlClass(string id)
        {
            ID = id;

            Parents = new OwlClassDictionary();
            Equivalents = new OwlClassDictionary();
            Complements = new OwlClassDictionary();
            Restrictions = new OwlRestrictionCollection();

            IsRemote = false;
        }

        public OwlClass(string id, bool isRemote) : this(id)
        {
            IsRemote = isRemote;
        }

        public override bool Equals(object o) 
        {
            if (!(o is OwlClass))
                return false;

            return ((OwlClass)o).ID.Equals(this.ID);
        }

        public override int GetHashCode()
        {
            return ID.GetHashCode();
        }
        
        public string RelativeID
        {
            get
            {
                if (ID.LastIndexOf('#') == -1)
                {
                    return ID;
                }
                else
                {
                    return ID.Substring(ID.LastIndexOf('#') + 1, ID.Length - ID.LastIndexOf('#') - 1);
                }
            }
        }

        public override string ToString()
        {
            System.Text.StringBuilder builder = new System.Text.StringBuilder();

            builder.Append("[OwlClass ID: " + ID + ", ");

            builder.Append(Parents.Count + " parents");
            if (Parents.Count > 0)
            {
                IDictionaryEnumerator e = Parents.GetEnumerator();
                e.MoveNext();
                builder.Append(" (" + ((OwlClass)e.Value).ID);
                if (Parents.Count > 1)
                    builder.Append(", ...)");
                else
                    builder.Append(")");
            }
            builder.Append(", ");

            builder.Append(Restrictions.Count + " restrictions");
            if (Restrictions.Count > 0)
            {
                IEnumerator e = Restrictions.GetEnumerator();
                e.MoveNext();
                builder.Append(" (" + e.Current);
                if (Restrictions.Count > 1)
                    builder.Append(", ...)");
                else
                    builder.Append(")");
            }

            builder.Append("]");

            return builder.ToString();
        }
    }
}
