using System;
using System.Collections;

using log4net;

using jade.core;
using jade.wrapper;
using jade.util.leap;

using java.util;

namespace Mailabs.Tutorial.MAS.Start
{
    public class Program
    {
        private static readonly string DEFAULT_CONFIG_FILE = "jadeleap.properties";
        private static readonly ILog logger;

        static Program()
        {
            log4net.Config.XmlConfigurator.Configure();
            logger = LogManager.GetLogger(typeof(Program));
        }

        private jade.wrapper.AgentContainer container;

        private Program() { }

        private void Start()
        {
            Profile profile = null;

            try
            {
                if (System.IO.File.Exists(DEFAULT_CONFIG_FILE))
                {
                    profile = new ProfileImpl(DEFAULT_CONFIG_FILE);
                    logger.Debug("Loaded profile from config file");
                }
                else
                {
                    logger.Error("Error while loading profile, please check availability of ./jadeleap.properties");
                    System.Environment.Exit(1);
                }
            }
            catch (ProfileException e)
            {
                logger.Error("Error while loading profile, please check availability and syntax of ./jadeleap.properties", e);
                System.Environment.Exit(1);
            }

            try
            {
                if (profile.getParameter("main", "true").Equals("false"))
                {
                    logger.Debug("Starting agent container...");
                    container = Runtime.instance().createAgentContainer(profile);
                    logger.Debug("Agent container started.");
                }
                else
                {
                    logger.Debug("Starting main container...");
                    container = Runtime.instance().createMainContainer(profile);
                    logger.Debug("Main container started.");
                }

                logger.Debug("Ready to instantiate agents");
                InstantiateAgents(DEFAULT_CONFIG_FILE);
            }
            catch (Exception e)
            {
                logger.Error("Something went wrong", e);
            }
        }

        private void InstantiateAgents(string propsFileName)
        {
            java.util.Properties props = new java.util.Properties();
            props.load(new java.io.FileInputStream(propsFileName));

            System.Collections.Hashtable agents = new System.Collections.Hashtable();
            for (Enumeration e = props.propertyNames(); e.hasMoreElements(); )
            {
                string propName = (string)e.nextElement();
                if (propName.StartsWith(Consts.AGENT_CONFIG_PREFIX))
                {
                    string agentName = propName.Substring(Consts.AGENT_CONFIG_PREFIX.Length, propName.LastIndexOf('.') - Consts.AGENT_CONFIG_PREFIX.Length);
                    string agentPropName = propName.Substring(propName.LastIndexOf('.') + 1);
                    string agentPropValue = props.getProperty(propName);

                    if (agentPropName.Length == 0)
                        continue;

                    logger.Debug("Found property '" + agentPropName + "' for agent '" + agentName + "', the value is '" + agentPropValue + "'");
                    if (!agents.Contains(agentName))
                    {
                        System.Collections.ArrayList agentProps = new System.Collections.ArrayList();
                        agents.Add(agentName, agentProps);
                    }

                    ((System.Collections.ArrayList)agents[agentName]).Add(agentPropName + "=" + agentPropValue);
                }
            }

            logger.Debug("Configuration fetched successfully, starting " + agents.Count + " agents");
            for (IDictionaryEnumerator e = agents.GetEnumerator(); e.MoveNext(); )
            {
                object[] agentProps = ((System.Collections.ArrayList)e.Value).ToArray();
                AgentController controller = container.createNewAgent((string)e.Key, "Mailabs.Tutorial.MAS.CommAgent", agentProps);
                controller.start();
                logger.Debug("Started agent: " + controller.getName());
            }
        }

        public static void Main(string[] args)
        {
            Program program = new Program();
            program.Start();
        }
    }
}
