using System;
using System.Collections;
using System.Xml;
using System.Text.RegularExpressions;
//using System.Windows.Forms;

namespace AQUARIUM
{
	/// <summary>
   /// v.1.0 (#5)
	///  .      
	///  ( ).
	/// </summary>
	public class XAttractors
	{


		/// <summary>
		///    XML-
		/// </summary>
		public void SaveInXML(string FileName)
		{
			XmlTextWriter writer = null;
			try
			{
				writer = new XmlTextWriter(FileName,System.Text.Encoding.Unicode);
				writer.Formatting = Formatting.Indented;

				writer.WriteStartDocument();
				writer.WriteStartElement("Attractors");
				XAttractor zAttractor;
				for (int i=0; i<Count(); i++)
				{
					zAttractor = this.GetAttractor(i);
					writer.WriteStartElement("Attractor");
					writer.WriteElementString("UIN", (zAttractor.m_UIN.ToString()));
					writer.WriteElementString("Radius", (zAttractor.m_Radius.ToString()));
					writer.WriteStartElement("Location");
					writer.WriteElementString("LocationX", (zAttractor.Location.x.ToString()));
					writer.WriteElementString("LocationY", (zAttractor.Location.y.ToString()));
					writer.WriteEndElement();
					writer.WriteStartElement("OldLocation");
					writer.WriteElementString("OldLocation", (zAttractor.OldLocation.x.ToString()));
					writer.WriteElementString("OldLocation", (zAttractor.OldLocation.x.ToString()));
					writer.WriteEndElement();
					for (int j=0; j<zAttractor.m_Tensity.m_Factor.Length; j++)
						writer.WriteElementString("Tensity"+j.ToString(), zAttractor.m_Tensity[j].ToString());
					writer.WriteEndElement();
				}
				writer.WriteEndElement();
			}
			finally
			{
				if (writer!=null) writer.Close();
			}
		}

		/// <summary>
		///    
		/// </summary>
		public void LoadFromXML(string FileName)
		{
			XmlTextReader reader = null;

			try
			{
				reader = new XmlTextReader(FileName);
				reader.WhitespaceHandling = WhitespaceHandling.None;
				foreach (XAttractor zAttractor in m_Attractors)
				{
					zAttractor.RIPDraw(Uplink.m_Canvas);
				}
					
				this.m_Attractors = new ArrayList();
				Regex regex = new Regex("^Tensity[0-9]{1,}$");
				Regex NumberReg = new Regex("Tensity");
				while (reader.Read())
				{
					if (reader.Name=="Attractor")
					{
						XAttractor zAttractor = new XAttractor(this,0,0);
						while ((reader.Read())&(reader.Name!="Attractor"))
						{
							switch (reader.Name)
							{
								case "UIN" :
									reader.Read();
									zAttractor.m_UIN = (Convert.ToUInt32((reader.Value)));
									reader.Read();
									break;
								case "Radius":
									reader.Read();
									zAttractor.m_Radius = Convert.ToInt32(reader.Value);
									reader.Read();
									break;
								case "LocationX":
									reader.Read();
									zAttractor.Location.x = Convert.ToInt32(reader.Value);
									reader.Read();
									break;
								case "LocationY":
									reader.Read();
									zAttractor.Location.y = Convert.ToInt32(reader.Value);
									reader.Read();
									break;
								
								default :
									if (regex.IsMatch(reader.Name))
									{
										string[] parts = NumberReg.Split(reader.Name);
										
										reader.Read();
										zAttractor.m_Tensity.m_Factor[Convert.ToInt32(reader.Value)]=Convert.ToInt32(parts[1]);
										reader.Read();
									}
									break;
							}
						}
						this.Add(zAttractor);
					}			
				}
			}
		    catch(Exception e)
			{
			}
			finally
			{
				if (reader!=null) reader.Close();
			}
		}



		/// <summary>
		///  
		/// </summary>
		private ArrayList m_Attractors;

      /// <summary>
      ///   
      /// </summary>
      public XEngine m_Uplink;

		public XEngine Uplink
		{
			get
			{
				return m_Uplink;
			}
		}


      /// <summary>
      ///   i- 
      /// </summary>
      /// <param name="i"></param>
      /// <returns>i- </returns>
      public XAttractor GetAttractor(int i)
      {
         return m_Attractors[i] as XAttractor;
      }


      /// <summary>
      ///  
      /// </summary>
      /// <param name="field"></param>
      public void Add(XAttractor field)
      {
         m_Attractors.Add(field);
      }

      /// <summary>
      ///    
      /// </summary>
      /// <param name="a_v"></param>
      /// <returns></returns>
      public int SearchByLocation(XVector a_v)
      {
         XAttractor zAttractor;
         int i=0;
         while (i<this.Count())
         {
            zAttractor=(m_Attractors[i])as XAttractor;
            double r=XVector.Distance(zAttractor.Location,a_v);
            if (r<=(zAttractor.m_Radius/m_Uplink.m_ScreenScale))
            {
               return (i);
            } 
            else i++;
         }
         return (-1);
      }

      /// <summary>
      ///  
      /// </summary>
      /// <param name="i"></param>
      public void Remove(int i)
      {
         m_Attractors.RemoveAt(i);
      }

      /// <summary>
      ///   
      /// </summary>
      /// <returns></returns>
      public int Count()
      {
         return(m_Attractors.Count);
      }

      /// <summary>
      /// 
      /// </summary>
		public XAttractors(XEngine a_Uplink)
		{
			m_Attractors=new ArrayList();
         m_Uplink = a_Uplink;
		}

      /// <summary>
      ///      
      /// </summary>
      /// <param name="v"></param>
      /// <returns>-  </returns>
		public XFactor Potential(XVector v)
      {
         XFactor potential=new XFactor();
         potential=XFactor.GetNull();
         for (int i=0; i<this.Count(); i++)
         {
            double d=100/XVector.Distance(v,(m_Attractors[i] as XAttractor).Location);
            potential=potential+(m_Attractors[i] as XAttractor).m_Tensity*(d);
         }
         return potential;
      }

      /// <summary>
      ///  
      /// </summary>
      public void Draw()
      {
         foreach (XAttractor zAttractor in m_Attractors)
         {
				if (Uplink.WasVisibleAttractor(zAttractor))
				{
					zAttractor.RIPOldDraw(m_Uplink.m_Canvas);
				}
				if (Uplink.IsVisibleAttractor(zAttractor))
				{
					
					zAttractor.Draw(m_Uplink.m_Canvas);
				}
         }
      }

		public void RIPDraw()
		{
			foreach (XAttractor zAttractor in m_Attractors)
			{
				if (Uplink.IsVisibleAttractor(zAttractor))
				{
					zAttractor.RIPOldDraw(m_Uplink.m_Canvas);
				}
			}
		}
	}
}
