using System;
using System.Collections;
using System.Drawing;
using System.IO;

namespace AQUARIUM
{
	/// <summary>
	/// v.1.0 (#4)
	///   ,  .
	/// </summary>
	public class XEngine
	{
		//  
		/// <summary>
		///   BT-
		/// </summary>
		public int m_BTRadius = 1000;

		/// <summary>
		/// 
		/// </summary>
		public double m_ScreenScale=0.1F;
		public float m_WorldScale=0.1F;

		/// <summary>
		///  
		/// </summary>
		public int m_Height = 45000;
		public int m_Width = 60000;
		public int m_VisibleHeight;
		public int m_VisibleWidth;

		/// <summary>
		///  
		/// </summary>
		public bool ShowGrid;

		/// <summary>
		///     .x - , .y -  
		/// </summary>
		public XVector m_GridDelta;

		/// <summary>
		///        
		/// </summary>
		public XVector m_LeftTop;

		/// <summary>
		///   (!!!)
		/// </summary>
		public Color m_BgColor;

		/// <summary>
		///   
		/// </summary>
		public XAgents m_Agents;

		/// <summary>
		///  
		/// </summary>
		public Graphics m_Canvas;

		/// <summary>
		///   
		/// </summary>
		public XAttractors m_Attractors;

		public XTypeList m_AgentTypes;

		public XTypeList m_AttractorTypes;

		/// <summary>
		///        
		/// </summary>
		/// <param name="x"></param>
		/// <returns></returns>
		public int ScrToWorldX(int x)
		{
			return (int)(x/m_ScreenScale) + m_LeftTop.x;
		}

		/// <summary>
		///        
		/// </summary>
		/// <param name="y"></param>
		/// <returns></returns>
		public int ScrToWorldY(int y)
		{
			return (int)(y/m_ScreenScale) + m_LeftTop.y;
		}

		/// <summary>
		///        
		/// </summary>
		/// <param name="x"></param>
		/// <returns></returns>
		public int WorldToScrX(int x)
		{
			return (int)( (x - m_LeftTop.x) );
		}

		/// <summary>
		///        
		/// </summary>
		/// <param name="y"></param>
		/// <returns></returns>
		public int WorldToScrY(int y)
		{
			return (int)( (y - m_LeftTop.y) );
		}

		public double CalculateScreenScale(int a_Width, int a_Height)
		{
			if ((double)a_Width/(double)a_Height>(double)m_Width/(double)m_Height)return a_Width/(m_Width*m_WorldScale);
			else return a_Height/(m_Height*m_WorldScale);
		}

		public double CalculateWorldScale(int a_Width, int a_Height)
		{
			if ((double)a_Width/(double)a_Height>(double)m_Width/(double)m_Height)return ((a_Width/m_ScreenScale)/m_Width);
			else return ((a_Height/m_ScreenScale)/m_Height);
		}

		public void ResetTransform(int a_Width, int a_Height)
		{
			m_Canvas.ResetTransform();
			m_WorldScale = 0.1F;
			m_ScreenScale = CalculateScreenScale(a_Width,a_Height);
			m_VisibleWidth = (int) (a_Width/m_ScreenScale);
			m_VisibleHeight = (int) (a_Height/m_ScreenScale);
			m_GridDelta = new XVector(m_Width/24,m_Height/24);
			m_Canvas.ScaleTransform((float)m_ScreenScale,(float) m_ScreenScale);
			m_LeftTop.x=0;
			m_LeftTop.y=0;
		}

		/// <summary>
		///  
		/// </summary>
		/// <param name="a_Canvas"></param>
		public XEngine(Graphics a_Canvas, int a_Width, int a_Height)
		{
			
			m_Agents=new XAgents(this);
			m_Attractors=new XAttractors(this);
			m_AgentTypes = new XTypeList();
			m_AttractorTypes = new XTypeList();
			m_AttractorTypes.Add("XOrdianryAttractor");
			m_Canvas = a_Canvas;
			m_LeftTop = new XVector();
			ResetTransform(a_Width,a_Height);

			TestInit();
		}

		/// <summary>
		///     
		/// </summary>
		public void Clear()
		{
			for (int i=m_Attractors.Count()-1; i>=0 ; i--)
			{
				m_Attractors.Remove(0);
			}

			for (int i=m_Agents.Count()-1; i>=0; i--)
			{
				m_Agents.Remove(0);
			}
			m_Canvas.Clear(m_BgColor);
		}

		/// <summary>
		///   
		/// </summary>
		private void TestInit()
		{
//			XOrdinaryAgent zAgent;
////			XThinkingAgent zTAgent;
////
////			zAgent = new XOrdinaryAgent(this.m_Agents,732,564,30,30);
////			m_Agents.Add(zAgent);
////
////			zTAgent = new XThinkingAgent(this.m_Agents,2500,2500,55,83);
////			this.m_Agents.Add(zTAgent);
////
////			zAgent = new XOrdinaryAgent(this.m_Agents,117,248,50,-20);
////			this.m_Agents.Add(zAgent);
//			zAgent = new XOrdinaryAgent(this.m_Agents);
//			m_Agents.Add(zAgent);
//			zAgent = new XOrdinaryAgent(this.m_Agents);
//			m_Agents.Add(zAgent);
//			zAgent = new XOrdinaryAgent(this.m_Agents);
//			m_Agents.Add(zAgent);
//
//
//			XAttractor zAttr;
//			zAttr = new XAttractor(m_Attractors);
//			this.m_Attractors.Add(zAttr);
		}

		/// <summary>
		///  
		/// </summary>
		public void NextTurn()
		{
			XAgent zAgent;
			for (int i=0; i<m_Agents.Count();i++)
			{
            
				zAgent=m_Agents.GetAgent(i);
				zAgent.Move();
				XVector zLocation = new XVector();
				XVector zSpeed = new XVector();
				XVector zTmpLoc;
				XVector zTmpSpd;
				zAgent.GetState(out zTmpLoc,out zTmpSpd);
				zLocation.x=zTmpLoc.x;
				zLocation.y=zTmpLoc.y;
				zSpeed.x=zTmpSpd.x;
				zSpeed.y=zTmpSpd.y;
				XVector zVect=zLocation+zSpeed;
				if (zVect.y+zAgent.m_Radius/m_ScreenScale>m_Height)
				{
					zLocation.y=(m_Height-(int)(zAgent.m_Radius/m_ScreenScale))*2-zLocation.y-zSpeed.y;
					zSpeed.y=-1*zSpeed.y;
				} 
				else if (zVect.y-zAgent.m_Radius/m_ScreenScale<0)
				{
					zLocation.y=(int)(2*zAgent.m_Radius/m_ScreenScale)-zLocation.y-zSpeed.y;
					zSpeed.y=-1*zSpeed.y;
				}
				else if (zVect.x+zAgent.m_Radius/m_ScreenScale>m_Width)
				{
					zLocation.x=2*(m_Width-(int)(zAgent.m_Radius/m_ScreenScale))-zLocation.x-zSpeed.x;
					zSpeed.x=-1*zSpeed.x;
				}
				else if (zVect.x<zAgent.m_Radius/m_ScreenScale)
				{
					zLocation.x=2*((int)(zAgent.m_Radius/m_ScreenScale))-zLocation.x-zSpeed.x;
					zSpeed.x=-1*zSpeed.x;
				}

				zAgent.SetState(zLocation+zSpeed,zSpeed);
			}
		}
			
		/// <summary>
		///  
		/// </summary>
		public void Draw()
		{
			if (ShowGrid) DrawGrid();
			m_Attractors.Draw();
			m_Agents.Draw();
		}

		public void RIPDraw()
		{
			RIPDrawGrid();
			m_Agents.RIPDraw();
			m_Attractors.RIPDraw();
		}


		/// <summary>
		///  
		/// </summary>
		public void DrawGrid()
		{
			DrawVerticalGrid(Color.LightGray);
			DrawHorizontalGrid(Color.LightGray); 
		}

		public void DrawHorizontalGrid(Color a_Color)
		{
         int y = m_GridDelta.y-m_LeftTop.y%m_GridDelta.y;
			Pen GreenPen = new Pen(a_Color,0);
			while ( y<=m_VisibleHeight)
			{
				m_Canvas.DrawLine(GreenPen,0,y,m_VisibleWidth,y);
				y+=m_GridDelta.y;
			}
			GreenPen.Dispose();
		}
		
		public void DrawVerticalGrid(Color a_Color)
		{
			int x = m_GridDelta.x-m_LeftTop.x%m_GridDelta.x ;
			Pen GreenPen = new Pen(a_Color,0);
			while ( x<=m_VisibleWidth)
			{
				m_Canvas.DrawLine(GreenPen,x,0,x,m_VisibleHeight);
				x+=m_GridDelta.x;
			}  
			GreenPen.Dispose();
		}

		public void RIPDrawGrid()
		{
			DrawVerticalGrid(m_BgColor);
			DrawHorizontalGrid(m_BgColor);
		}


		

		/// <summary>
		///   
		/// </summary>
		public void LogFileSave()
		{
			try
			{
				FileStream LogFile = new FileStream("Log.txt", FileMode.Append);
				StreamWriter Logstr_out = new StreamWriter(LogFile);	
				Logstr_out.Write(m_Agents.LogStr());
				Logstr_out.Close();	
			}
			catch (System.Exception ex)
			{
				Console.WriteLine("Error: " + ex.Message);
			}
		}

		/// <summary>
		///       
		/// </summary>
		/// <param name="v"></param>
		/// <returns></returns>
		public bool IsVisibleAgent(XAgent zAgent)
		{
			int zDelta=(int)(2*zAgent.m_Radius/m_ScreenScale);
			XVector v = new XVector(zAgent.Location);
			if ( (v.x>m_LeftTop.x-zDelta)&&(v.x<m_LeftTop.x+m_VisibleWidth+zDelta)&&(v.y>m_LeftTop.y-zDelta)&&(v.y<m_LeftTop.y+m_VisibleHeight+zDelta) ) return true;
			else return false;
		}


		/// <summary>
		///       
		/// </summary>
		/// <param name="v"></param>
		/// <returns></returns>
		public bool IsVisibleAttractor(XAttractor zAttractor)
		{
			XVector v = new XVector(zAttractor.Location);
			int zDelta=(int)(2*zAttractor.m_Radius/m_ScreenScale);
			if ( (v.x>m_LeftTop.x-zDelta)&&(v.x<m_LeftTop.x+m_VisibleWidth+zDelta)&&(v.y>m_LeftTop.y-zDelta)&&(v.y<m_LeftTop.y+m_VisibleHeight+zDelta) ) return true;
			else return false;
		}


		/// <summary>
		///           
		/// </summary>
		/// <param name="zAgent"></param>
		/// <returns></returns>
		public bool WasVisibleAgent(XAgent zAgent)
		{
			int zDelta=(int)(2*zAgent.m_Radius/m_ScreenScale);
			XVector v = new XVector(zAgent.OldLocation);
			if ( (v.x>m_LeftTop.x-zDelta)&&(v.x<m_LeftTop.x+m_VisibleWidth+zDelta)&&(v.y>m_LeftTop.y-zDelta)&&(v.y<m_LeftTop.y+m_VisibleHeight+zDelta) ) return true;
			else return false;
		}

		/// <summary>
		///           
		/// </summary>
		/// <param name="zAttractor"></param>
		/// <returns></returns>
		public bool WasVisibleAttractor(XAttractor zAttractor)
		{
			XVector v = new XVector(zAttractor.OldLocation);
			int zDelta=(int)(2*zAttractor.m_Radius/m_ScreenScale);
			if ( (v.x>m_LeftTop.x-zDelta)&&(v.x<m_LeftTop.x+m_VisibleWidth+zDelta)&&(v.y>m_LeftTop.y-zDelta)&&(v.y<m_LeftTop.y+m_VisibleHeight+zDelta) ) return true;
			else return false;
		}

		public XVector CorrectLeftTop(int x, int y)
		{
			if (x+m_VisibleWidth>m_Width)
			{
				x=m_Width-m_VisibleWidth;
			} else
			if (x<0) 
			{
				x=0;
			};

			if (y+m_VisibleHeight>m_Height)
			{
				y=m_Height-m_VisibleHeight;
			} 
			else
			if (y<0) 
			{
				y=0;
			};
			return new XVector(x,y);
		}

		public void CenterBy(XVector a_Center)
		{
			int x,y;
			x=a_Center.x - m_VisibleWidth/2;
			y=a_Center.y - m_VisibleHeight/2;
         XVector vect = CorrectLeftTop(x,y);
			m_LeftTop.x=vect.x;
			m_LeftTop.y=vect.y;
		}
	}
}